/**
 * Version 1.1.14
 */

(function( $ ) {
    'use strict';


    /**
     * Modal Components
     */
    if ( typeof( window.ELV_MODAL) === 'undefined') {
        window.ELV_MODAL = null;
    }

    /**
     * API Components
     */
    window.ElvPostMixin = {
        methods: {
            post: function(url, params, success_callback=null, failure_callback=null) {
                axios.post(url, params)
                .then( function(res) {
                    if ( success_callback ) {
                        // Apply callback function. Args should be into array.
                        this[success_callback].apply(this, [res]);
                    }
                }.bind(this))
                .catch( function(res) {
                    if ( failure_callback ) {
                        // Apply callback function. Args should be into array.
                        this[failure_callback].apply(this, [res]);
                    }
                }.bind(this))
                .finally( function() {
                    /**
                     * TODO: add callback function
                     */
                }.bind(this))
            },
            ajax_post( api, args, success_callback=null, failure_callback=null) {
                var params =  new FormData();
                Object.keys(args).forEach( function(key) {
                    params.append( key, args[key] );
                });

                params.append('action', api.action);
                params.append('nonce', api.nonce);

                this.post(api.api, params, success_callback, failure_callback);
            },
            is_post_success(res) {
                if ( res.data && res.data.result ) {
                    const result = res.data.result;
                    if (result == 'SUCCESS') {
                        return true;
                    } else {
                        return false;
                    }
                } else {
                    return false;
                }
            }
        }
    }
    /**
     * TTS Components
     */
    window.ElvTTSMixin = {
        data() {
            return {
                speech_icon_class: 'fas fa-volume-up animate__animated animate__flash animate__slower animate__infinite',
            }
        },
        methods: {
            /**
             *
             * @param {text: String} args
             */
            speech(args={}) {
                if (typeof ELVEZ_TTS_AJAX != 'undefined') {
                    const api = ELVEZ_TTS_AJAX;
                    this.ajax_post(api, args, 'speech_success');
                }
            },
            speech_success(res) {
                const text_base64 = res.data.speech;
                this.startSpeech(text_base64);
            },
            startSpeech(text_base64) {
                this.stopSpeech();
                const src = ['data:audio/mp3;base64,' + text_base64];
                const sound = new Howl({src: src});

                window.TTSSound = sound;
                TTSSound.play();

                $(window).trigger('start.tts.speech');
                TTSSound.on('end', function() {
                    $(window).trigger('stop.tts.speech');
                })
            },
            stopSpeech() {
                if (typeof TTSSound != 'undefined') {
                    TTSSound.stop();
                    $(window).trigger('stop.tts.speech');
                }
            },
            show_speech_icon() {
                const $icon = $('.elv-modal .modal-header .header-left-icon');
                $icon.addClass( this.speech_icon_class );
                $icon.removeClass( 'd-none' );
            },
            hide_speech_icon() {
                const $icon = $('.elv-modal .modal-header .header-left-icon');
                $icon.removeClass( this.speech_icon_class );
                $icon.addClass( 'd-none' );
            },
        },
        mounted() {
            $(window).on('start.tts.speech', this.show_speech_icon);
            $(window).on('stop.tts.speech', this.hide_speech_icon);
        },
    }


    var ElvModalTemplate = `
        <div id="elv-modal" class="elv-modal modal my-auto"
                :class=" modal.centered ? 'animate__animated animate__fadeIn animate__fast' : 'fade modal-bottom'"
                tabindex="-1" role="dialog" aria-labelledby="modalDialogTitle" aria-hidden="true"
                data-backdrop="static" data-keyboard="false">
            <div class="modal-dialog" role="document"
                :class=" modal.centered ? 'modal-dialog-centered' : null ">
                <div class="modal-content elv-rounded"
                    :class="modal.content_class">
                    <div v-if="is_show_header"
                        class="modal-header border-bottom">
                        <component :is=header_component :modal="modal"></component>
                    </div>

                    <div v-if="modal.tabs" class="modal-body border-bottom">
                        <component :is=tab_component :modal="modal"></component>
                    </div>

                    <div class="modal-body">
                        <component :is=modal.body_component :modal="modal"></component>
                    </div>
                    <div class="modal-footer">
                        <component :is=footer_component :modal="modal"></component>
                    </div>
                </div>
            </div>
        <div
    `;
    var ElvModal = Vue.extend({
        mixins: [ElvPostMixin, ElvTTSMixin],
        data() {
            return {
                id: 'elv-modal',
                instance: null,
                modal: {},
                is_show_header: true,
            }
        },
        computed: {
            tab_component() {
                return this.modal.tab_component ? this.modal.tab_component : 'elv_modal_tab';
            },
            header_component() {
                return this.modal.header_component ? this.modal.header_component : 'elv_modal_header';
            },
            footer_component() {
                return this.modal.footer_component ? this.modal.footer_component : 'elv_modal_footer';
            }
        },
        methods: {
            show(callback=null) {
                if ( callback ) {
                    $(this.$el).on('shown.bs.modal', callback).modal('show');
                } else {
                    $(this.$el).modal('show');
                }
            },
            hide(callback=null) {
                $(this.$el).modal('hide');
                $('body').removeClass('modal-open');
                $('.modal-backdrop').remove();
            },
            on_show() {
                if ( this.modal.speech_text) {
                    const args = {
                        text: this.modal.speech_text,
                    }
                    this.speech(args);
                }
            }
        },
        mounted(){
            $(this.$el).on('show.bs.modal', this.on_show);
        }
    });

    Vue.component('elv_modal_tab', {
        props: {
            modal: Object,
        },
        data() {
            return {}
        },
        template: `
        <div class="elv-modal-tab-wrapper">
            <button v-for="tab in modal.tabs"
                type="button"
                class="elv-modal-tab-button elv-rounded p-0">
                <span>{{ tab.button_text }}</span>
            </button>
        </div>
        `
    })

    Vue.component('elv_modal_header', {
        props: {
            modal: Object,
        },
        data() {
            return {}
        },
        template: `
            <div class="elv-w-100">
                <!--
                <div class="btn close-icon" data-dismiss="modal">
                    <i class="far fa-times-circle"></i>
                </div>
                -->
                <div class="header-left-icon-wrapper">
                    <i class="header-left-icon my-1 d-none"></i>
                </div>
                <div class="modal-title text-center w-100 font-weight-bold my-1">
                    <p class="mb-0">{{ modal.title }}</p>
                </div>
            </div>

        `
    })

    /**
     * Modal footer includes submit, close, back and action butons.
     *
     * @params button text
     * close_button_text
     * submit_button_text
     * back_button_text
     * action_button_text
     *
     * @params forms
     * submit_form  : trriger form submit event on click submit button.
     *
     * @params callback functions
     * on_close( modal, event )
     * on_back( modal, event )
     * on_action( modal, event )
     *
     */
    Vue.component('elv_modal_footer', {
        props: {
            modal: Object,
        },
        data() {
            return {
                close_button_text_default: 'とじる',
                submit_button_text_default: '決定',
                back_button_text_default: '戻る',
                action_button_text_default: '',
            }
        },
        computed: {
            close_button_text() {
                return this.modal.close_button_text ? this.modal.close_button_text : this.close_button_text_default;
            },
            submit_button_text() {
                return this.modal.submit_button_text ? this.modal.submit_button_text : this.submit_button_text_default;
            },
            back_button_text() {
                return this.modal.back_button_text ? this.modal.back_button_text : this.back_button_text_default;
            },
            action_button_text() {
                return this.modal.action_button_text ? this.modal.action_button_text : this.action_button_text_default;
            },
            button_animation_class() {
                return 'animate__animated animate__pulse animate__slow animate__infinite';
            }
        },
        template: `
        <div class="elv-modal-footer-wrapper">
            <template v-if="!(false === modal.show_back_button)">
                <button v-if="modal.submit_form || modal.on_submit || modal.show_back_button"
                    type="button"
                    data-dismiss="modal"
                    class="elv-modal-back-button elv-rounded p-0"
                    v-on:click="modal.on_back ? modal.on_back( modal ) : null;">
                    <span>{{ back_button_text }}</span>
                </button>
            </template>
            <template v-if="!(false === modal.show_submit_button)">
                <button v-if="modal.submit_form"
                    type="submit"
                    :form="modal.submit_form"
                    :class="button_animation_class"
                    class="elv-modal-submit-button elv-rounded mx-auto p-0">
                    <span class="">{{ submit_button_text }}</span>
                </button>
                <button v-else-if="modal.on_submit"
                    type="button"
                    data-dismiss="modal"
                    :class="button_animation_class"
                    class="elv-modal-submit-button elv-rounded mx-auto p-0"
                    v-on:click="modal.on_submit();">
                    <span class="">{{ submit_button_text }}</span>
                </button>
                <button v-else-if="!(false === modal.show_close_button)"
                    type="button"
                    data-dismiss="modal"
                    class="elv-modal-close-button elv-rounded mx-auto p-0"
                    :class="button_animation_class"
                    v-on:click="modal.on_close ? modal.on_close( modal ) : null;">
                    <span class="">{{ close_button_text }}</span>
                </button>
            </template>
            <button v-if="action_button_text"
                type="button"
                class="elv-modal-action-button elv-rounded p-0"
                :class="modal.action_button_class"
                v-on:click="modal.on_action ? modal.on_action( modal ) : null;">
                <span>{{ action_button_text }}</span>
            </button>
        </div>
        `
    })

    window.ElvModalMixin = {
        data() {
            return {
                'modal_instance': null,
                'modal': {},
                'modal_stack': [],
            }
        },
        methods: {
            create_modal( modal={} ){
                /**
                 * スタックが有効なら現在のモーダルデータをスタックに入れる
                 * 新しいインスタンスを取得してモーダルデータを設定
                 */
                const stack_previous = typeof( modal.stack_previous ) != 'undefined' ? modal.stack_previous : true;
                const stackable = typeof( modal.stackable ) != 'undefined' ? modal.stackable : true;

                if ( stack_previous && typeof( window.ELV_MODAL) != 'undefined' && window.ELV_MODAL && window.ELV_MODAL.stackable ) {
                    const prev_modal = window.ELV_MODAL.prev_modal ? window.ELV_MODAL.prev_modal : window.ELV_MODAL.modal;
                    this.modal_stack.push( prev_modal );
                }

                this.modal_instance = this.get_modal_instance();
                this.modal_instance.modal = modal;
                this.modal_instance.stackable = stackable;

            },
            show_modal( modal={} ) {
                /**
                 * スタックが有効なら現在のモーダルデータをスタックに入れる
                 * 新しいインスタンスを取得してモーダルデータを設定
                 * モーダルを表示
                 */
                this.create_modal( modal );
                this.modal_instance.show( modal.on_shown );
            },
            show_prev_modal() {
                /**
                 * スタックから最後の1件を取得
                 * スタックのモーダルを渡して表示
                 */
                const prev_modal = this.modal_stack.pop();
                if ( prev_modal ) {
                    prev_modal['stack_previous'] = false;
                    this.show_modal( prev_modal );
                    return true;
                } else {
                    return false;
                }
            },
            hide_modal() {
                this.modal_instance.hide();
            },
            clear_modal_stack() {
                this.modal_stack = [];
            },
            get_modal_instance() {
                if ( typeof( window.ELV_MODAL) === 'undefined' || window.ELV_MODAL == null ) {
                    return this.__create_modal();
                } else {
                    /**
                     * Create new isntance.
                     * And keep previous modal data as prev_modal.
                     */
                    window.ELV_MODAL.hide();
                    $('#elv-modal').remove();
                    return this.__create_modal();
                }
            },
            __create_modal() {
                /**
                 * Create new html elment.
                 * And update current modal instance.
                 */
                $('body').append(ElvModalTemplate);
                const $el = $('#elv-modal');
                const instance = new ElvModal().$mount($el[0]);
                window.ELV_MODAL = instance;
                return instance;
            },
            remove_modal() {
            },
        },
    };


    /**
     * Common Components
     */
    Vue.component('elvez_modal_message_modal_body', {
        props: {
            modal: Object,
        },
        data() {
            return {}
        },
        template: `
            <div class="elv-text-center">
                <p v-if="modal.subtitle"
                    class="elv-message-modal-subtitle">{{ modal.subtitle }}</p>
                <div v-if="modal.image"
                    class="elv-card-media-area">
                    <img class="mx-auto"
                        :src="modal.image"/>
                </div>
                <p class="elv-message-modal-message">{{ modal.message }}</p>
            </div>
        `
    })


})( jQuery );